@@ -252,7 +252,7 @@ rrtype_descriptor_type rrtype_descriptors[RRTYPE_DESCRIPTORS_LENGTH] = {
 	/* 43 */
 	{ TYPE_DS, "DS", T_DS, 4, 4,
 	  { RDATA_WF_SHORT, RDATA_WF_BYTE, RDATA_WF_BYTE, RDATA_WF_BINARY },
-	  { RDATA_ZF_SHORT, RDATA_ZF_BYTE, RDATA_ZF_BYTE, RDATA_ZF_HEX } },
+	  { RDATA_ZF_SHORT, RDATA_ZF_ALGORITHM, RDATA_ZF_BYTE, RDATA_ZF_HEX } },
 	/* 44 */
 	{ TYPE_SSHFP, "SSHFP", T_SSHFP, 3, 3,
 	  { RDATA_WF_BYTE, RDATA_WF_BYTE, RDATA_WF_BINARY },
@@ -264,7 +264,7 @@ rrtype_descriptor_type rrtype_descriptors[RRTYPE_DESCRIPTORS_LENGTH] = {
 	  { RDATA_WF_SHORT, RDATA_WF_BYTE, RDATA_WF_BYTE, RDATA_WF_LONG,
 	    RDATA_WF_LONG, RDATA_WF_LONG, RDATA_WF_SHORT,
 	    RDATA_WF_UNCOMPRESSED_DNAME, RDATA_WF_BINARY },
-	  { RDATA_ZF_RRTYPE, RDATA_ZF_BYTE, RDATA_ZF_BYTE, RDATA_ZF_PERIOD,
+	  { RDATA_ZF_RRTYPE, RDATA_ZF_ALGORITHM, RDATA_ZF_BYTE, RDATA_ZF_PERIOD,
 	    RDATA_ZF_TIME, RDATA_ZF_TIME, RDATA_ZF_SHORT, RDATA_ZF_DNAME,
 	    RDATA_ZF_BASE64 } },
 	/* 47 */
